/** EMULib Emulation Library *********************************/
/**                                                         **/
/**                         ComUnix.c                       **/
/**                                                         **/
/** This file contains standard communication routines for  **/
/** Unix using TCP/IP via sockets.                          **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1997-1998                 **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/
#ifdef UNIX

#include "ComUnix.h"

#include <strings.h>
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

/** Connect() ************************************************/
/** Establish TCP/IP connection. When Server=0, this side   **/
/** will immediately become a server. Otherwise, it will    **/
/** first try to connect to a given server as a client and  **/
/** become a server if this fails. Returns -1 on failure,   **/
/** or socket handle otherwise.                             **/
/*************************************************************/
int Connect(const char *Server,unsigned short Port)
{
  struct sockaddr_in Addr;
  struct hostent *Host;
  int Socket,AddrLength,Socket1;

  /* Clear the address structure */
  memset(&Addr,0,sizeof(Addr));

  /* If the server name is given, we first try to */
  /* connect as a client.                         */
  if(Server)
  {
    /* Look up server name */
    if(!(Host=gethostbyname(Server))) return(-1);

    /* Set fields of the address structure */
    memcpy(&Addr.sin_addr,Host->h_addr,Host->h_length);
    Addr.sin_family = AF_INET;
    Addr.sin_port   = htons(Port);

    /* Create a socket */
    if((Socket=socket(AF_INET,SOCK_STREAM,0))<0) return(-1);

    /* Connecting... */
    if(connect(Socket,(struct sockaddr *)&Addr,sizeof(Addr))>=0)
    {
      /* Succesfully connected as client */
      return(Socket);
    }

    /* Failed to connect as a client */
    close(Socket);
  }

  /* Connection as client either failed or hasn't */
  /* been attempted at all. Becoming a server and */
  /* waiting for connection request.              */

  /* Set fields of the address structure */
  Addr.sin_addr.s_addr = htonl(INADDR_ANY);
  Addr.sin_family      = AF_INET;
  Addr.sin_port        = htons(Port);

  /* Create a socket */
  if((Socket=socket(AF_INET,SOCK_STREAM,0))<0) return(-1);

  /* Bind socket */
  if(bind(Socket,(struct sockaddr *)&Addr,sizeof(Addr))<0)
  { close(Socket);return(-1); }

  /* Listen for two connects, incoming and outgoing */
  listen(Socket,2);

  /* Accepting calls... */
  AddrLength=sizeof(Addr);
  Socket1=accept(Socket,(struct sockaddr *)&Addr,&AddrLength);
  close(Socket);

  /* Client failed to connect */
  if(Socket1<0) return(-1);

  /* Client connected succesfully */
  return(Socket1);
}

/** RecvByte() ***********************************************/
/** Receive a single byte from a socket. Returns 0 on       **/
/** failure, or 1 on success.                               **/
/*************************************************************/
int RecvByte(int Socket,unsigned char *Value)
{ return(recv(Socket,Value,1,0)==1); }

/** SendByte() ***********************************************/
/** Send a single byte to a socket. Returns 0 on failure,   **/
/** or 1 on success.                                        **/
/*************************************************************/
int SendByte(int Socket,unsigned char Value)
{ return(send(Socket,&Value,1,0)==1); }

#endif /* UNIX */
